import { ItemStack, EquipmentSlot, GameMode, world } from "@minecraft/server";

/**
 * @param {number} min The minimum integer
 * @param {number} max The maximum integer
 * @returns {number} A random integer between the `min` and `max` parameters (inclusive)
 */
const randomInt = (min, max) =>
    Math.floor(Math.random() * (max - min + 1)) + min;

const maxGrowth = 7;


/** @type {import("@minecraft/server").BlockCustomComponent} */
const CustomCropGrowthBlockComponent = {
    onRandomTick({ block }) {
        const growth = block.permutation.getState("coffee:growth");
        if (growth == 7)
            return;
        const growthChance = 1 / 2.5;
        if (Math.random() > growthChance) return;

        block.setPermutation(
            block.permutation.withState("coffee:growth", growth + 1)
        );
    },
    onPlayerDestroy(event) {

        const growth = event.destroyedBlockPermutation.getState("coffee:growth");
        if (growth < 7)
            return;
        event.dimension.spawnEntity("minecraft:xp_orb", event.block.location);
        const mainHand = event.player.getComponent("minecraft:equippable").getEquipment("Mainhand");
        if (mainHand == null || !mainHand.hasTag("minecraft:is_hoe")) {
            return;
        }
        const enchantable = mainHand.getComponent("minecraft:enchantable");
        const fortune = enchantable?.getEnchantment("fortune");

        const fortune_level = fortune?.level ?? 0;
        const random = Math.round(Math.random() * fortune_level + 0.5);
        for (let i = 0; i < random; i++) {
            event.dimension.spawnItem(new ItemStack("coffee:coffee_cherries", 1), event.block.location);
            event.dimension.spawnEntity("minecraft:xp_orb", event.block.location);

        }

    },
    onPlayerInteract({ block, dimension, player }) {
        if (!player) return;
        let growth = block.permutation.getState("coffee:growth");
        if (growth == 7)
            return;

        const equippable = player.getComponent("minecraft:equippable");
        if (!equippable) return;

        const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
        if (!mainhand.hasItem() || mainhand.typeId !== "minecraft:bone_meal")
            return;

        if (player.getGameMode() === GameMode.creative) {
            // Grow crop fully
            block.setPermutation(block.permutation.withState("coffee:growth", 7));
        } else {

            // Add random amount of growth
            growth += randomInt(1, maxGrowth - growth);
            block.setPermutation(
                block.permutation.withState("coffee:growth", growth)
            );

            // Decrement stack
            if (mainhand.amount > 1) mainhand.amount--;
            else mainhand.setItem(undefined);
        }

        // Play effects
        const effectLocation = block.center();
        dimension.playSound("item.bone_meal.use", effectLocation);
        dimension.spawnParticle(
            "minecraft:crop_growth_emitter",
            effectLocation
        );
    },
};

world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {

    blockComponentRegistry.registerCustomComponent(
        "coffee:custom_crop_growth",
        CustomCropGrowthBlockComponent
    );
});
