import "./custom_crop";
import { Block, ItemStack, system, world } from '@minecraft/server'

const cost_coffee1 = 2;
const cost_coffee2 = 5;

world.beforeEvents.playerBreakBlock.subscribe((event) => {
    if (event.player.getGameMode() == "creative")
        return;
    if (event.block.typeId == "coffee:coffee_maker") {
        const remaining = getRemaining(event.block.permutation.getState("coffee:remaining"),
            event.block.permutation.getState("coffee:div"));
        if (event.player.getGameMode() != "creative" && remaining > 0) {
            system.run(() => {

                event.dimension.spawnItem(new ItemStack("coffee:powder", remaining), event.block.location);
            });
        }
    }
    if (event.block.typeId == "minecraft:short_grass" || event.block.typeId == "minecraft:tall_grass") {
        const random = Math.random();
        if (random > 0.05)
            return;
        system.run(() => {

            event.dimension.spawnItem(new ItemStack("coffee:coffee_cherries", 1), event.block.location);
        });
    }
})

world.afterEvents.itemCompleteUse.subscribe((event) => {
    const typeId = event.itemStack.typeId;
    if (!event.itemStack.hasTag("coffee:coffee")) {
        return;
    }
    const player = event.source;
    const time1 = 40 * 20;
    const time2 = 100 * 20

    let currentStack = player.getDynamicProperty("coffee:stack");
    if (currentStack == undefined)
        currentStack = 0;



    if (event.itemStack.typeId == "coffee:coffee1") {
        player.addEffect("minecraft:speed", time1, { "showParticles": false });
        player.addEffect("haste", time1, { "showParticles": false });
    }
    else if (event.itemStack.typeId == "coffee:coffee2") {
        const options = { "amplifier": 2, "showParticles": false };
        player.addEffect("minecraft:speed", time2, options);
        player.addEffect("haste", time2, options);
    }
})

/** @type {import("@minecraft/server").BlockCustomComponent} */
const BlockCustomComponent = {

    onPlayerInteract(event) {
        const player = event.player;
        const equippable = player.getComponent("minecraft:equippable");
        const mainHand = equippable.getEquipment("Mainhand");
        if (mainHand == undefined) {
            return;
        }
        let remaining = getRemaining(event.block.permutation.getState("coffee:remaining"),
            event.block.permutation.getState("coffee:div"));

        const mainHandTypeId = mainHand.typeId;
        if (mainHandTypeId == "coffee:powder") {
            if (remaining < 50)
                event.player.dimension.playSound("bucket.empty_water", event.player.location);

            for (let i = 0; i < 5; i++) {
                if (remaining >= 50 || mainHand == null)
                    break;
                reduceItem(player, 1);
                remaining += 1;

            }


            setRemaining(event.block, remaining);
            //     const perm = event.block.permutation.withState("coffee:div", Math.trunc(remaining / 10));
            //     const finalPerm = perm.withState("coffee:remaining", remaining % 10);
        }
        else if (mainHandTypeId == "coffee:cup" || mainHandTypeId == "coffee:mug") {

            let remaining = getRemaining(event.block.permutation.getState("coffee:remaining"),
                event.block.permutation.getState("coffee:div"));

            if (mainHandTypeId == "coffee:cup") {
                if (remaining >= cost_coffee1) {
                    player.runCommand("give @s coffee:coffee1");
                    reduceItem(event.player, 1);
                    setRemaining(event.block, remaining - cost_coffee1);
                    event.player.dimension.playSound("bucket.fill_water", event.player.location);
                }
                else {

                    event.player.sendMessage(`§lYou need at least ${cost_coffee1} units of coffee powder `);
                }
            }
            else if (mainHandTypeId == "coffee:mug") {
                if (remaining >= cost_coffee2) {
                    player.runCommand("give @s coffee:coffee2");
                    reduceItem(event.player, 1);
                    event.player.dimension.playSound("bucket.fill_water", event.player.location);

                    setRemaining(event.block, remaining - cost_coffee2);
                }
                else {
                    event.player.sendMessage(`§lYou need at least ${cost_coffee2} units of coffee powder `);

                }
            }
        }
        getRemaining(event.block.permutation.getState("coffee:div"));
    }

};



world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
    blockComponentRegistry.registerCustomComponent("coffee:coffee_maker", BlockCustomComponent);
});

world.afterEvents.worldInitialize.subscribe((event) => {
    world.getDimension("minecraft:overworld").runCommand("gamerule sendcommandfeedback false");

})

function setRemaining(block, value) {
    if (value <= 0) {
        finalPerm = finalPerm.withState("coffee:fullness", 0);
        block.setPermutation(0);
        return;
    }
    const perm = block.permutation.withState("coffee:div", Math.trunc(value / 10));
    let finalPerm = perm.withState("coffee:remaining", value % 10);
    let fullness = 0;
    const remaining = getRemaining(block.permutation.getState("coffee:remaining"),
        block.permutation.getState("coffee:div"));
    if (remaining >= cost_coffee1)
        fullness = 1;
    if (remaining >= 25)
        fullness = 2;
    if (remaining >= 50)
        fullness = 3;

    finalPerm = finalPerm.withState("coffee:fullness", fullness);
    block.setPermutation(finalPerm);
}

function getRemaining(remaining, div) {
    const dec = div * 10
    return remaining + dec
}

system.runInterval(() => {
    //  world.getAllPlayers()[0].addEffect("minecraft:speed", 10);
});


function reduceItem(player, quantity) {
    if (player.getGameMode() == "creative")
        return;
    const equip = player.getComponent("minecraft:equippable");
    let item = equip.getEquipment("Mainhand");
    if (item == null)
        return;
    if (item.amount <= quantity)
        item = null;
    else
        item.amount -= quantity;
    equip.setEquipment("Mainhand", item);
}